@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2002-2007 Keisuke Nishida
- * Copyright (C) 2007 Roger While
+ * Copyright (C) 2002-2009 Keisuke Nishida
+ * Copyright (C) 2007-2009 Roger While
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -29,9 +29,9 @@
 #include "move.h"
 #include "coblocal.h"
 #include "byteswap.h"
-/*
-#include "lib/gettext.h"
-*/
+
+static size_t		lastsize = 0;
+static unsigned char	*lastdata = NULL;
 
 static const int	cob_exp10[10] = {
 	1,
@@ -92,57 +92,111 @@ own_byte_memcpy (unsigned char *s1, const unsigned char *s2, size_t size)
 	do {
 		*s1++ = *s2++;
 	} while (--size);
-/*
-	while (size--) {
-		*s1++ = *s2++;
-	}
-	switch (size) {
-	case 8:
-		*s1++ = *s2++;
-	case 7:
-		*s1++ = *s2++;
-	case 6:
-		*s1++ = *s2++;
-	case 5:
-		*s1++ = *s2++;
-	case 4:
-		*s1++ = *s2++;
-	case 3:
-		*s1++ = *s2++;
-	case 2:
-		*s1++ = *s2++;
-	case 1:
-		*s1++ = *s2++;
-		return;
-	default:
-		return;
-	}
-*/
 }
 
 static void
 store_common_region (cob_field *f, const unsigned char *data,
 		     const size_t size, const int scale)
 {
-	int	lf1 = -scale;
-	int	lf2 = -COB_FIELD_SCALE (f);
-	int	hf1 = (int) size + lf1;
-	int	hf2 = (int) COB_FIELD_SIZE (f) + lf2;
-	int	lcf;
-	int	gcf;
+	const unsigned char	*p;
+	unsigned char		*q;
+	size_t			csize;
+	size_t			cinc;
+	int			lf1 = -scale;
+	int			lf2 = -COB_FIELD_SCALE (f);
+	int			hf1 = (int) size + lf1;
+	int			hf2 = (int) COB_FIELD_SIZE (f) + lf2;
+	int			lcf;
+	int			gcf;
 
 	lcf = cob_max_int (lf1, lf2);
 	gcf = cob_min_int (hf1, hf2);
+	memset (COB_FIELD_DATA (f), '0', COB_FIELD_SIZE (f));
 	if (gcf > lcf) {
-		own_memset (COB_FIELD_DATA (f), '0', (size_t)(hf2 - gcf));
-		own_memcpy (COB_FIELD_DATA (f) + hf2 - gcf, data + hf1 - gcf,
-				(size_t)(gcf - lcf));
-		own_memset (COB_FIELD_DATA (f) + hf2 - lcf, '0', (size_t)(lcf - lf2));
-	} else {
-		own_memset (f->data, '0', f->size);
+		csize = (size_t)(gcf - lcf);
+		p = data + hf1 - gcf;
+		q = COB_FIELD_DATA (f) + hf2 - gcf;
+		for (cinc = 0; cinc < csize; ++cinc, ++p, ++q) {
+			if (unlikely(*p == ' ')) {
+				*q = (unsigned char)'0';
+			} else {
+				*q = *p;
+			}
+		}
 	}
 }
 
+static long long
+cob_binary_mget_int64 (const cob_field * const f)
+{
+	long long	n = 0;
+	size_t		fsiz = 8 - f->size;
+
+/* Experimental code - not activated */
+#if 0
+	unsigned char	*s;
+
+	if ((COB_FIELD_BINARY_SWAP (f) && !COB_FIELD_HAVE_SIGN (f)) ||
+	    (!COB_FIELD_BINARY_SWAP (f) && COB_FIELD_HAVE_SIGN (f))) {
+		s = (unsigned char *)&n + fsiz;
+	} else {
+		s = (unsigned char *)&n;
+	}
+	own_byte_memcpy (s, f->data, f->size);
+	if (COB_FIELD_BINARY_SWAP (f)) {
+		n = COB_BSWAP_64 (n);
+	}
+	if (COB_FIELD_HAVE_SIGN (f)) {
+		n >>= 8 * fsiz;	/* shift with sign */
+	}
+#endif
+#ifndef WORDS_BIGENDIAN
+	if (COB_FIELD_BINARY_SWAP (f)) {
+		if (COB_FIELD_HAVE_SIGN (f)) {
+			own_byte_memcpy ((unsigned char *)&n, f->data, f->size);
+			n = COB_BSWAP_64 (n);
+			n >>= 8 * fsiz;	/* shift with sign */
+		} else {
+			own_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
+			n = COB_BSWAP_64 (n);
+		}
+	} else {
+		if (COB_FIELD_HAVE_SIGN (f)) {
+			own_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
+			n >>= 8 * fsiz;	/* shift with sign */
+		} else {
+			own_byte_memcpy ((unsigned char *)&n, f->data, f->size);
+		}
+	}
+#else	/* WORDS_BIGENDIAN */
+	if (COB_FIELD_HAVE_SIGN (f)) {
+		own_byte_memcpy ((unsigned char *)&n, f->data, f->size);
+		n >>= 8 * fsiz;	/* shift with sign */
+	} else {
+		own_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
+	}
+#endif	/* WORDS_BIGENDIAN */
+	return n;
+}
+
+static void
+cob_binary_mset_int64 (cob_field *f, long long n)
+{
+#ifndef WORDS_BIGENDIAN
+	unsigned char	*s;
+
+	if (COB_FIELD_BINARY_SWAP (f)) {
+		n = COB_BSWAP_64 (n);
+		s = ((unsigned char *)&n) + 8 - f->size;
+	} else {
+		s = (unsigned char *)&n;
+	}
+	own_byte_memcpy (f->data, s, f->size);
+#else	/* WORDS_BIGENDIAN */
+	own_byte_memcpy (f->data, ((unsigned char *)&n) + 8 - f->size, f->size);
+#endif	/* WORDS_BIGENDIAN */
+}
+
 /*
  * Display
  */
@@ -150,19 +204,23 @@ store_common_region (cob_field *f, const unsigned char *data,
 static void
 cob_move_alphanum_to_display (cob_field *f1, cob_field *f2)
 {
-	int		sign, count, size;
 	unsigned char	*p;
-	unsigned char	*s1 = f1->data;
-	unsigned char	*s2 = COB_FIELD_DATA (f2);
-	unsigned char	*e1 = s1 + f1->size;
-	unsigned char	*e2 = s2 + COB_FIELD_SIZE (f2);
+	unsigned char	*s1;
+	unsigned char	*s2;
+	unsigned char	*e1;
+	unsigned char	*e2;
+	int		sign, count, size;
 	unsigned char	c;
 
 	/* initialize */
-	own_memset (f2->data, '0', f2->size);
+	s1 = f1->data;
+	e1 = s1 + f1->size;
+	s2 = COB_FIELD_DATA (f2);
+	e2 = s2 + COB_FIELD_SIZE (f2);
+	memset (f2->data, '0', f2->size);
 
 	/* skip white spaces */
-	for (; s1 < e1; s1++) {
+	for (; s1 < e1; ++s1) {
 		if (!isspace (*s1)) {
 			break;
 		}
@@ -170,15 +228,17 @@ cob_move_alphanum_to_display (cob_field *f1, cob_field *f2)
 
 	/* check for sign */
 	sign = 0;
-	if (*s1 == '+' || *s1 == '-') {
-		sign = (*s1++ == '+') ? 1 : -1;
+	if (s1 != e1) {
+		if (*s1 == '+' || *s1 == '-') {
+			sign = (*s1++ == '+') ? 1 : -1;
+		}
 	}
 
 	/* count the number of digits before decimal point */
 	count = 0;
-	for (p = s1; p < e1 && *p != cob_current_module->decimal_point; p++) {
+	for (p = s1; p < e1 && *p != cob_current_module->decimal_point; ++p) {
 		if (isdigit (*p)) {
-			count++;
+			++count;
 		}
 	}
 
@@ -196,7 +256,7 @@ cob_move_alphanum_to_display (cob_field *f1, cob_field *f2)
 
 	/* move */
 	count = 0;
-	for (; s1 < e1 && s2 < e2; s1++) {
+	for (; s1 < e1 && s2 < e2; ++s1) {
 		c = *s1;
 		if (isdigit (c)) {
 			*s2++ = c;
@@ -213,15 +273,16 @@ cob_move_alphanum_to_display (cob_field *f1, cob_field *f2)
 	return;
 
 error:
-	own_memset (f2->data, '0', f2->size);
+	memset (f2->data, '0', f2->size);
 	cob_put_sign (f2, 0);
 }
 
 static void
 cob_move_display_to_display (cob_field *f1, cob_field *f2)
 {
-	int	sign = cob_get_sign (f1);
+	int	sign;
 
+	sign = cob_get_sign (f1);
 	store_common_region (f2, COB_FIELD_DATA (f1), COB_FIELD_SIZE (f1),
 			     COB_FIELD_SCALE (f1));
 
@@ -232,29 +293,34 @@ cob_move_display_to_display (cob_field *f1, cob_field *f2)
 static void
 cob_move_display_to_alphanum (cob_field *f1, cob_field *f2)
 {
+	unsigned char	*data1;
+	unsigned char	*data2;
+	size_t		size1;
+	size_t		size2;
+	int		sign;
 	int		diff;
 	int		zero_size;
-	int		sign = cob_get_sign (f1);
-	size_t		size1 = COB_FIELD_SIZE (f1);
-	size_t		size2 = f2->size;
-	unsigned char	*data1 = COB_FIELD_DATA (f1);
-	unsigned char	*data2 = f2->data;
 
+	data1 = COB_FIELD_DATA (f1);
+	size1 = COB_FIELD_SIZE (f1);
+	sign = cob_get_sign (f1);
+	data2 = f2->data;
+	size2 = f2->size;
 	if (size1 >= size2) {
-		own_memcpy (data2, data1, size2);
+		memcpy (data2, data1, size2);
 	} else {
 		diff = (int)(size2 - size1);
 		zero_size = 0;
 		/* move */
-		own_memcpy (data2, data1, size1);
+		memcpy (data2, data1, size1);
 		/* implied 0 ('P's) */
 		if (COB_FIELD_SCALE(f1) < 0) {
 			zero_size = cob_min_int ((int)-COB_FIELD_SCALE(f1), diff);
-			own_memset (data2 + size1, '0', zero_size);
+			memset (data2 + size1, '0', (size_t)zero_size);
 		}
 		/* padding */
 		if (diff - zero_size > 0) {
-			own_memset (data2 + size1 + zero_size, ' ', diff - zero_size);
+			memset (data2 + size1 + zero_size, ' ', (size_t)(diff - zero_size));
 		}
 	}
 
@@ -264,26 +330,30 @@ cob_move_display_to_alphanum (cob_field *f1, cob_field *f2)
 static void
 cob_move_alphanum_to_alphanum (cob_field *f1, cob_field *f2)
 {
-	size_t		size1 = f1->size;
-	size_t		size2 = f2->size;
-	unsigned char	*data1 = f1->data;
-	unsigned char	*data2 = f2->data;
+	unsigned char	*data1;
+	unsigned char	*data2;
+	size_t		size1;
+	size_t		size2;
 
+	data1 = f1->data;
+	size1 = f1->size;
+	data2 = f2->data;
+	size2 = f2->size;
 	if (size1 >= size2) {
 		/* move string with truncation */
 		if (COB_FIELD_JUSTIFIED (f2)) {
-			own_memcpy (data2, data1 + size1 - size2, size2);
+			memcpy (data2, data1 + size1 - size2, size2);
 		} else {
-			own_memcpy (data2, data1, size2);
+			memcpy (data2, data1, size2);
 		}
 	} else {
 		/* move string with padding */
 		if (COB_FIELD_JUSTIFIED (f2)) {
-			own_memset (data2, ' ', size2 - size1);
-			own_memcpy (data2 + size2 - size1, data1, size1);
+			memset (data2, ' ', size2 - size1);
+			memcpy (data2 + size2 - size1, data1, size1);
 		} else {
-			own_memcpy (data2, data1, size1);
-			own_memset (data2 + size1, ' ', size2 - size1);
+			memcpy (data2, data1, size1);
+			memset (data2 + size1, ' ', size2 - size1);
 		}
 	}
 }
@@ -295,23 +365,36 @@ cob_move_alphanum_to_alphanum (cob_field *f1, cob_field *f2)
 static void
 cob_move_display_to_packed (cob_field *f1, cob_field *f2)
 {
+	unsigned char	*data1;
+	unsigned char	*data2;
+	unsigned char	*p;
+	size_t		digits1;
+	size_t		digits2;
 	size_t		i;
 	size_t		offset;
-	int		sign = cob_get_sign (f1);
-	size_t		digits1 = COB_FIELD_DIGITS (f1);
-	size_t		digits2 = COB_FIELD_DIGITS (f2);
-	int		scale1 = COB_FIELD_SCALE (f1);
-	int		scale2 = COB_FIELD_SCALE (f2);
-	unsigned char	*data1 = COB_FIELD_DATA (f1);
-	unsigned char	*data2 = f2->data;
-	unsigned char	*p = data1 + (digits1 - scale1) - (digits2 - scale2);
+	int		sign;
+	int		scale1;
+	int		scale2;
 	unsigned char	n;
 
+	sign = cob_get_sign (f1);
+	data1 = COB_FIELD_DATA (f1);
+	digits1 = COB_FIELD_DIGITS (f1);
+	scale1 = COB_FIELD_SCALE (f1);
+	data2 = f2->data;
+	digits2 = COB_FIELD_DIGITS (f2);
+	scale2 = COB_FIELD_SCALE (f2);
+
 	/* pack string */
-	own_memset (f2->data, 0, f2->size);
+	memset (f2->data, 0, f2->size);
 	offset = 1 - (digits2 % 2);
-	for (i = offset; i < digits2 + offset; i++, p++) {
-		n = (data1 <= p && p < data1 + digits1) ? cob_d2i (*p) : 0;
+	p = data1 + (digits1 - scale1) - (digits2 - scale2);
+	for (i = offset; i < digits2 + offset; ++i, ++p) {
+		if (*p == ' ') {
+			n = 0;
+		} else {
+			n = (data1 <= p && p < data1 + digits1) ? cob_d2i (*p) : 0;
+		}
 		if (i % 2 == 0) {
 			data2[i / 2] = n << 4;
 		} else {
@@ -320,25 +403,30 @@ cob_move_display_to_packed (cob_field *f1, cob_field *f2)
 	}
 
 	cob_put_sign (f1, sign);
+	p = f2->data + f2->size - 1;
 	if (!COB_FIELD_HAVE_SIGN (f2)) {
-		data2[digits2 / 2] |= 0x0f;
+		*p = (*p & 0xf0) | 0x0f;
+	} else if (sign < 0) {
+		*p = (*p & 0xf0) | 0x0d;
 	} else {
-		cob_real_put_sign (f2, sign);
+		*p = (*p & 0xf0) | 0x0c;
 	}
 }
 
 static void
 cob_move_packed_to_display (cob_field *f1, cob_field *f2)
 {
+	unsigned char	*data;
 	size_t		i;
 	size_t		offset;
-	int		sign = cob_get_sign (f1);
-	unsigned char	*data = f1->data;
+	int		sign;
 	unsigned char	buff[64];
 
 	/* unpack string */
+	data = f1->data;
+	sign = cob_get_sign (f1);
 	offset = 1 - (COB_FIELD_DIGITS(f1) % 2);
-	for (i = offset; i < COB_FIELD_DIGITS(f1) + offset; i++) {
+	for (i = offset; i < COB_FIELD_DIGITS(f1) + offset; ++i) {
 		if (i % 2 == 0) {
 			buff[i - offset] = cob_i2d (data[i / 2] >> 4);
 		} else {
@@ -366,7 +454,7 @@ cob_move_display_to_fp (cob_field *f1, cob_field *f2)
 	char		*data1;
 	char		buff2[64];
 
-	own_memset ((ucharptr)buff2, 0, sizeof (buff2));
+	memset ((ucharptr)buff2, 0, sizeof (buff2));
 	size = size1 - COB_FIELD_SCALE(f1);
 	if (sign < 0) {
 		buff2[0] = '-';
@@ -375,9 +463,9 @@ cob_move_display_to_fp (cob_field *f1, cob_field *f2)
 		data1 = buff2;
 	}
 	if (COB_FIELD_SCALE(f1) <= 0) {
-		sprintf (data1, "%*.*s.0", (int)size, (int)size, f1->data);
+		snprintf (data1, 63, "%*.*s.0", (int)size, (int)size, f1->data);
 	} else {
-		sprintf (data1, "%*.*s.%*.*s", (int)size, (int)size, f1->data,
+		snprintf (data1, 63, "%*.*s.%*.*s", (int)size, (int)size, f1->data,
 			 COB_FIELD_SCALE(f1), COB_FIELD_SCALE(f1), f1->data + size);
 	}
 	sscanf (buff2, "%lf", &val);
@@ -403,8 +491,8 @@ cob_move_fp_to_display (cob_field *f1, cob_field *f2)
 	char		buff[64];
 	char		buff2[64];
 
-	own_memset ((ucharptr)buff, 0, sizeof (buff));
-	own_memset ((ucharptr)buff2, 0, sizeof (buff2));
+	memset ((ucharptr)buff, 0, sizeof (buff));
+	memset ((ucharptr)buff2, 0, sizeof (buff2));
 	if (COB_FIELD_TYPE (f1) == COB_TYPE_NUMERIC_FLOAT) {
 		float	flval;
 
@@ -422,11 +510,11 @@ cob_move_fp_to_display (cob_field *f1, cob_field *f2)
 	res = (long long) intgr;
 	decs = 0;
 	for (; res; res /= 10) {
-		decs++;
+		++decs;
 	}
-	sprintf (buff2, "%-18.*lf", 18 - decs, val);
+	snprintf (buff2, 63, "%-18.*lf", 18 - decs, val);
 	y = buff;
-	for (x = buff2; *x; x++) {
+	for (x = buff2; *x; ++x) {
 		if (*x == '.') {
 			continue;
 		}
@@ -447,12 +535,15 @@ cob_move_fp_to_display (cob_field *f1, cob_field *f2)
 static void
 cob_move_display_to_binary (cob_field *f1, cob_field *f2)
 {
+	unsigned char	*data1;
 	size_t		i, size;
+	size_t		size1;
 	long long	val = 0;
-	int		sign = cob_get_sign (f1);
-	size_t		size1 = COB_FIELD_SIZE (f1);
-	unsigned char	*data1 = COB_FIELD_DATA (f1);
+	int		sign;
 
+	size1 = COB_FIELD_SIZE (f1);
+	data1 = COB_FIELD_DATA (f1);
+	sign = cob_get_sign (f1);
 	/* get value */
 	size = size1 - COB_FIELD_SCALE(f1) + COB_FIELD_SCALE(f2);
 	for (i = 0; i < size; ++i) {
@@ -471,7 +562,7 @@ cob_move_display_to_binary (cob_field *f1, cob_field *f2)
 	}
 
 	/* store */
-	cob_binary_set_int64 (f2, val);
+	cob_binary_mset_int64 (f2, val);
 
 	cob_put_sign (f1, sign);
 }
@@ -487,7 +578,7 @@ cob_move_binary_to_display (cob_field *f1, cob_field *f2)
 	sign = 1;
 	/* get value */
 	if (COB_FIELD_HAVE_SIGN (f1)) {
-		val2 = cob_binary_get_int64 (f1);
+		val2 = cob_binary_mget_int64 (f1);
 		if (val2 < 0) {
 			sign = -1;
 			val = -val2;
@@ -495,7 +586,7 @@ cob_move_binary_to_display (cob_field *f1, cob_field *f2)
 			val = val2;
 		}
 	} else {
-		val = cob_binary_get_int64 (f1);
+		val = cob_binary_mget_int64 (f1);
 	}
 
 	/* convert to string */
@@ -521,9 +612,9 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 {
 	const char	*p;
 	unsigned char	*min, *max, *src, *dst, *end;
-	unsigned char	*decimal_point = NULL;
-	int		sign = cob_get_sign (f1);
-	int		neg = (sign < 0) ? 1 : 0;
+	unsigned char	*decimal_point;
+	int		sign;
+	int		neg;
 	int		count = 0;
 	int		count_sign = 1;
 	int		count_curr = 1;
@@ -541,6 +632,9 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 	unsigned char	sign_symbol = 0;
 	unsigned char	curr_symbol = 0;
 
+	decimal_point = NULL;
+	sign = cob_get_sign (f1);
+	neg = (sign < 0) ? 1 : 0;
 	/* count the number of digit places before decimal point */
 	for (p = COB_FIELD_PIC (f2); *p; p += 5) {
 		c = p[0];
@@ -576,7 +670,7 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 		c = *p++;	/* PIC char */
 		memcpy ((unsigned char *)&n, p, sizeof(int));	/* PIC char count */
 		p += sizeof(int);
-		for (; n > 0; n--, dst++) {
+		for (; n > 0; n--, ++dst) {
 			switch (c) {
 			case '0':
 			case '/':
@@ -589,8 +683,8 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 
 			case 'P':
 				if (p_is_left) {
-					src++;
-					dst--;
+					++src;
+					--dst;
 				}
 				break;
 
@@ -605,7 +699,7 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 				break;
 
 			case 'V':
-				dst--;
+				--dst;
 				decimal_point = dst;
 				break;
 
@@ -645,12 +739,12 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 				}
 				if (trailing_sign) {
 					*dst = neg ? '-' : (c == '+') ? '+' : ' ';
-					end--;
+					--end;
 				} else if (dst == f2->data || suppress_zero) {
 					*dst = pad;
 					sign_symbol = neg ? '-' : (c == '+') ? '+' : ' ';
 					if (!curr_symbol) {
-						sign_first++;
+						++sign_first;
 					}
 				} else {
 					*dst = x;
@@ -665,7 +759,7 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 					}
 					if (trailing_curr) {
 						*dst = cob_current_module->currency_symbol;
-						end--;
+						--end;
 					} else if (dst == f2->data || suppress_zero) {
 						*dst = pad;
 						curr_symbol = cob_current_module->currency_symbol;
@@ -683,9 +777,9 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 	if (suppress_zero || (is_zero && COB_FIELD_BLANK_ZERO (f2))) {
 		/* all digits are zeros */
 		if (pad == ' ' || COB_FIELD_BLANK_ZERO (f2)) {
-			own_memset (f2->data, ' ', f2->size);
+			memset (f2->data, ' ', f2->size);
 		} else {
-			for (dst = f2->data; dst < f2->data + f2->size; dst++) {
+			for (dst = f2->data; dst < f2->data + f2->size; ++dst) {
 				if (*dst != cob_current_module->decimal_point) {
 					*dst = pad;
 				}
@@ -694,7 +788,7 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 	} else {
 		/* put zero after the decimal point if necessary */
 		if (decimal_point) {
-			for (dst = decimal_point + 1; dst < end; dst++) {
+			for (dst = decimal_point + 1; dst < end; ++dst) {
 				if (!isdigit (*dst) && !strchr (",+-/B", *dst)) {
 					*dst = '0';
 				}
@@ -703,7 +797,7 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 
 		/* put sign or currency symbol at the beginning */
 		if (sign_symbol || curr_symbol) {
-			for (dst = end - 1; dst > f2->data; dst--) {
+			for (dst = end - 1; dst > f2->data; --dst) {
 				if (*dst == ' ') {
 					break;
 				}
@@ -711,13 +805,13 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 			if (sign_symbol && curr_symbol) {
 				if (sign_first) {
 					*dst = curr_symbol;
-					dst--;
+					--dst;
 					if (dst >= f2->data) {
 						*dst = sign_symbol;
 					}
 				} else {
 					*dst = sign_symbol;
-					dst--;
+					--dst;
 					if (dst >= f2->data) {
 						*dst = curr_symbol;
 					}
@@ -731,7 +825,7 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 
 		/* replace all 'B's by pad */
 		count = 0;
-		for (dst = f2->data; dst < end; dst++) {
+		for (dst = f2->data; dst < end; ++dst) {
 			if (*dst == 'B') {
 				if (count == 0) {
 					*dst = pad;
@@ -739,7 +833,7 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 					*dst = ' ';
 				}
 			} else {
-				count++;
+				++count;
 			}
 		}
 	}
@@ -750,6 +844,8 @@ cob_move_display_to_edited (cob_field *f1, cob_field *f2)
 static void
 cob_move_edited_to_display (cob_field *f1, cob_field *f2)
 {
+	unsigned char	*p;
+	const char	*p1;
 	size_t		i;
 	int		sign = 0;
 	int		scale = 0;
@@ -757,14 +853,12 @@ cob_move_edited_to_display (cob_field *f1, cob_field *f2)
 	int		have_point = 0;
 	int		cp;
 	int		n;
-	unsigned char	*p;
-	const char	*p1;
 	unsigned char	c;
 	unsigned char	buff[64];
 
 	p = buff;
 	/* de-edit */
-	for (i = 0; i < f1->size; i++) {
+	for (i = 0; i < f1->size; ++i) {
 		cp = f1->data[i];
 		switch (cp) {
 		case '0':
@@ -779,7 +873,7 @@ cob_move_edited_to_display (cob_field *f1, cob_field *f2)
 		case '9':
 			*p++ = cp;
 			if (have_point) {
-				scale++;
+				++scale;
 			}
 			break;
 		case '.':
@@ -840,7 +934,7 @@ cob_move_alphanum_to_edited (cob_field *f1, cob_field *f2)
 		c = *p++;	/* PIC char */
 		memcpy ((unsigned char *)&n, p, sizeof(int));	/* PIC char count */
 		p += sizeof(int);
-		for (; n > 0; n--) {
+		for (; n > 0; --n) {
 			switch (c) {
 			case 'A':
 			case 'X':
@@ -890,8 +984,6 @@ cob_move_all (cob_field *src, cob_field *dst)
 	size_t			digcount;
 	cob_field		temp;
 	cob_field_attr		attr;
-	static size_t		lastsize = 0;
-	static unsigned char	*data = NULL;
 
 	COB_ATTR_INIT (COB_TYPE_ALPHANUMERIC, 0, 0, 0, NULL);
 	if (COB_FIELD_IS_NUMERIC(dst)) {
@@ -908,29 +1000,19 @@ cob_move_all (cob_field *src, cob_field *dst)
 	} else {
 		digcount = dst->size;
 	}
-	if (!data) {
-		if (digcount <= COB_SMALL_BUFF) {
-			data = cob_malloc (COB_SMALL_BUFF);
-			lastsize = COB_SMALL_BUFF;
-		} else {
-			data = cob_malloc (digcount);
-			lastsize = digcount;
-		}
-	} else {
-		if (digcount > lastsize) {
-			free (data);
-			data = cob_malloc (digcount);
-			lastsize = digcount;
-		}
+	if (digcount > lastsize) {
+		free (lastdata);
+		lastdata = cob_malloc (digcount);
+		lastsize = digcount;
 	}
 	temp.size = digcount;
-	temp.data = data;
+	temp.data = lastdata;
 	temp.attr = &attr;
 	if (likely(src->size == 1)) {
-		own_memset (data, src->data[0], (size_t)digcount);
+		memset (lastdata, src->data[0], digcount);
 	} else {
-		for (i = 0; i < digcount; i++) {
-			data[i] = src->data[i % src->size];
+		for (i = 0; i < digcount; ++i) {
+			lastdata[i] = src->data[i % src->size];
 		}
 	}
 
@@ -944,6 +1026,12 @@ cob_move (cob_field *src, cob_field *dst)
 		cob_move_all (src, dst);
 		return;
 	}
+	if (dst->size == 0) {
+		return;
+	}
+	if (src->size == 0) {
+		src = &cob_space;
+	}
 
 	/* non-elementary move */
 	if (COB_FIELD_TYPE (src) == COB_TYPE_GROUP || COB_FIELD_TYPE (dst) == COB_TYPE_GROUP) {
@@ -1072,14 +1160,42 @@ cob_move (cob_field *src, cob_field *dst)
 static int
 cob_packed_get_int (cob_field *f1)
 {
+	unsigned char	*data;
 	size_t		i;
 	size_t		offset;
 	int		val = 0;
-	int		sign = cob_get_sign (f1);
-	unsigned char	*data = f1->data;
+	int		sign;
 
+	data = f1->data;
+	sign = cob_get_sign (f1);
 	offset = 1 - (COB_FIELD_DIGITS(f1) % 2);
-	for (i = offset; i < COB_FIELD_DIGITS(f1) + offset; i++) {
+	for (i = offset; i < COB_FIELD_DIGITS(f1) - COB_FIELD_SCALE(f1) + offset; ++i) {
+		val *= 10;
+		if (i % 2 == 0) {
+			val += data[i / 2] >> 4;
+		} else {
+			val += data[i / 2] & 0x0f;
+		}
+	}
+	if (sign < 0) {
+		val = -val;
+	}
+	return val;
+}
+
+static long long
+cob_packed_get_long_long (cob_field *f1)
+{
+	unsigned char	*data;
+	size_t		i;
+	size_t		offset;
+	long long	val = 0;
+	int		sign;
+
+	data = f1->data;
+	sign = cob_get_sign (f1);
+	offset = 1 - (COB_FIELD_DIGITS(f1) % 2);
+	for (i = offset; i < COB_FIELD_DIGITS(f1) - COB_FIELD_SCALE(f1) + offset; ++i) {
 		val *= 10;
 		if (i % 2 == 0) {
 			val += data[i / 2] >> 4;
@@ -1096,14 +1212,17 @@ cob_packed_get_int (cob_field *f1)
 static int
 cob_display_get_int (cob_field *f)
 {
+	unsigned char	*data;
+	size_t		size;
 	size_t		i;
 	int		val = 0;
-	int		sign = cob_get_sign (f);
-	size_t		size = COB_FIELD_SIZE (f);
-	unsigned char	*data = COB_FIELD_DATA (f);
+	int		sign;
 
+	size = COB_FIELD_SIZE (f);
+	data = COB_FIELD_DATA (f);
+	sign = cob_get_sign (f);
 	/* skip preceding zeros */
-	for (i = 0; i < size; i++) {
+	for (i = 0; i < size; ++i) {
 		if (cob_d2i (data[i]) != 0) {
 			break;
 		}
@@ -1129,124 +1248,43 @@ cob_display_get_int (cob_field *f)
 	return val;
 }
 
-unsigned long long
-cob_binary_get_uint64 (const cob_field * const f)
+static long long
+cob_display_get_long_long (cob_field *f)
 {
-	unsigned long long	n = 0;
-	size_t			fsiz = 8 - f->size;
+	unsigned char	*data;
+	size_t		size;
+	size_t		i;
+	long long	val = 0;
+	int		sign;
 
-#ifndef WORDS_BIGENDIAN
-	if (COB_FIELD_BINARY_SWAP (f)) {
-		own_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
-		n = COB_BSWAP_64 (n);
-	} else {
-		own_byte_memcpy ((unsigned char *)&n, f->data, f->size);
-	}
-#else	/* WORDS_BIGENDIAN */
-	own_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
-#endif	/* WORDS_BIGENDIAN */
-	return n;
-}
-
-long long
-cob_binary_get_int64 (const cob_field * const f)
-{
-	long long	n = 0;
-	size_t		fsiz = 8 - f->size;
-
-/* Experimental code - not activated */
-#if 0
-	unsigned char	*s;
-
-	if ((COB_FIELD_BINARY_SWAP (f) && !COB_FIELD_HAVE_SIGN (f)) ||
-	    (!COB_FIELD_BINARY_SWAP (f) && COB_FIELD_HAVE_SIGN (f))) {
-		s = (unsigned char *)&n + fsiz;
-	} else {
-		s = (unsigned char *)&n;
-	}
-	own_byte_memcpy (s, f->data, f->size);
-	if (COB_FIELD_BINARY_SWAP (f)) {
-		n = COB_BSWAP_64 (n);
-	}
-	if (COB_FIELD_HAVE_SIGN (f)) {
-		n >>= 8 * fsiz;	/* shift with sign */
-	}
-#endif
-#ifndef WORDS_BIGENDIAN
-	if (COB_FIELD_BINARY_SWAP (f)) {
-		if (COB_FIELD_HAVE_SIGN (f)) {
-			own_byte_memcpy ((unsigned char *)&n, f->data, f->size);
-			n = COB_BSWAP_64 (n);
-			n >>= 8 * fsiz;	/* shift with sign */
-		} else {
-			own_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
-			n = COB_BSWAP_64 (n);
-		}
-	} else {
-		if (COB_FIELD_HAVE_SIGN (f)) {
-			own_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
-			n >>= 8 * fsiz;	/* shift with sign */
-		} else {
-			own_byte_memcpy ((unsigned char *)&n, f->data, f->size);
+	size = COB_FIELD_SIZE (f);
+	data = COB_FIELD_DATA (f);
+	sign = cob_get_sign (f);
+	/* skip preceding zeros */
+	for (i = 0; i < size; ++i) {
+		if (cob_d2i (data[i]) != 0) {
+			break;
 		}
 	}
-#else	/* WORDS_BIGENDIAN */
-	if (COB_FIELD_HAVE_SIGN (f)) {
-		own_byte_memcpy ((unsigned char *)&n, f->data, f->size);
-		n >>= 8 * fsiz;	/* shift with sign */
+
+	/* get value */
+	if (COB_FIELD_SCALE(f) < 0) {
+		for (; i < size; ++i) {
+			val = val * 10 + cob_d2i (data[i]);
+		}
+		val *= cob_exp10LL[(int)-COB_FIELD_SCALE(f)];
 	} else {
-		own_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
+		size -= COB_FIELD_SCALE(f);
+		for (; i < size; ++i) {
+			val = val * 10 + cob_d2i (data[i]);
+		}
 	}
-#endif	/* WORDS_BIGENDIAN */
-	return n;
-}
-
-int
-cob_binary_get_int (const cob_field * const f)
-{
-	return (int) cob_binary_get_int64 (f);
-}
-
-void
-cob_binary_set_int64 (cob_field *f, long long n)
-{
-#ifndef WORDS_BIGENDIAN
-	unsigned char	*s;
-
-	if (COB_FIELD_BINARY_SWAP (f)) {
-		n = COB_BSWAP_64 (n);
-		s = ((unsigned char *)&n) + 8 - f->size;
-	} else {
-		s = (unsigned char *)&n;
+	if (sign < 0) {
+		val = -val;
 	}
-	own_byte_memcpy (f->data, s, f->size);
-#else	/* WORDS_BIGENDIAN */
-	own_byte_memcpy (f->data, ((unsigned char *)&n) + 8 - f->size, f->size);
-#endif	/* WORDS_BIGENDIAN */
-}
 
-void
-cob_binary_set_uint64 (cob_field *f, unsigned long long n)
-{
-#ifndef WORDS_BIGENDIAN
-	unsigned char	*s;
-
-	if (COB_FIELD_BINARY_SWAP (f)) {
-		n = COB_BSWAP_64 (n);
-		s = ((unsigned char *)&n) + 8 - f->size;
-	} else {
-		s = (unsigned char *)&n;
-	}
-	own_byte_memcpy (f->data, s, f->size);
-#else	/* WORDS_BIGENDIAN */
-	own_byte_memcpy (f->data, ((unsigned char *)&n) + 8 - f->size, f->size);
-#endif	/* WORDS_BIGENDIAN */
-}
-
-void
-cob_binary_set_int (cob_field *f, int n)
-{
-	cob_binary_set_int64 (f, n);
+	cob_put_sign (f, sign);
+	return val;
 }
 
 void
@@ -1273,7 +1311,7 @@ cob_get_int (cob_field *f)
 	case COB_TYPE_NUMERIC_DISPLAY:
 		return cob_display_get_int (f);
 	case COB_TYPE_NUMERIC_BINARY:
-		return cob_binary_get_int (f);
+		return (int)cob_binary_mget_int64 (f);
 	case COB_TYPE_NUMERIC_PACKED:
 		return cob_packed_get_int (f);
 	default:
@@ -1286,3 +1324,35 @@ cob_get_int (cob_field *f)
 		return n;
 	}
 }
+
+long long
+cob_get_long_long (cob_field *f)
+{
+	long long	n;
+	cob_field	temp;
+	cob_field_attr	attr;
+
+	switch (COB_FIELD_TYPE (f)) {
+	case COB_TYPE_NUMERIC_DISPLAY:
+		return cob_display_get_long_long (f);
+	case COB_TYPE_NUMERIC_BINARY:
+		return cob_binary_mget_int64 (f);
+	case COB_TYPE_NUMERIC_PACKED:
+		return cob_packed_get_long_long (f);
+	default:
+		COB_ATTR_INIT (COB_TYPE_NUMERIC_BINARY, 18, 0,
+				COB_FLAG_HAVE_SIGN, NULL);
+		temp.size = 8;
+		temp.data = (unsigned char *)&n;
+		temp.attr = &attr;
+		cob_move (f, &temp);
+		return n;
+	}
+}
+
+void
+cob_init_move (void)
+{
+	lastdata = cob_malloc (COB_SMALL_BUFF);
+	lastsize = COB_SMALL_BUFF;
+}
