@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2001-2007 Keisuke Nishida
- * Copyright (C) 2007 Roger While
+ * Copyright (C) 2001-2009 Keisuke Nishida
+ * Copyright (C) 2007-2009 Roger While
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -57,6 +57,7 @@ static const unsigned char packed_bytes[] = {
 	0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99
 };
 
+static unsigned char	*num_buff_ptr;
 static cob_decimal	cob_d1;
 static cob_decimal	cob_d2;
 static cob_decimal	cob_d3;
@@ -93,8 +94,8 @@ mpz_set_ull (mpz_ptr dest, const unsigned long long val)
 static void
 mpz_set_sll (mpz_ptr dest, const signed long long val)
 {
-	size_t			size;
 	unsigned long long	vtmp;
+	size_t			size;
 
 	vtmp = (unsigned long long)(val >= 0 ? val : -val);
 	size = (vtmp != 0);
@@ -152,6 +153,122 @@ mpz_get_sll (const mpz_ptr src)
 
 #endif	/* COB_EXPERIMENTAL */
 
+static COB_INLINE void
+num_byte_memcpy (unsigned char *s1, const unsigned char *s2, size_t size)
+{
+	do {
+		*s1++ = *s2++;
+	} while (--size);
+}
+
+static long long
+cob_binary_get_int64 (const cob_field * const f)
+{
+	long long	n = 0;
+	size_t		fsiz = 8 - f->size;
+
+/* Experimental code - not activated */
+#if 0
+	unsigned char	*s;
+
+	if ((COB_FIELD_BINARY_SWAP (f) && !COB_FIELD_HAVE_SIGN (f)) ||
+	    (!COB_FIELD_BINARY_SWAP (f) && COB_FIELD_HAVE_SIGN (f))) {
+		s = (unsigned char *)&n + fsiz;
+	} else {
+		s = (unsigned char *)&n;
+	}
+	num_byte_memcpy (s, f->data, f->size);
+	if (COB_FIELD_BINARY_SWAP (f)) {
+		n = COB_BSWAP_64 (n);
+	}
+	if (COB_FIELD_HAVE_SIGN (f)) {
+		n >>= 8 * fsiz;	/* shift with sign */
+	}
+#endif
+#ifndef WORDS_BIGENDIAN
+	if (COB_FIELD_BINARY_SWAP (f)) {
+		if (COB_FIELD_HAVE_SIGN (f)) {
+			num_byte_memcpy ((unsigned char *)&n, f->data, f->size);
+			n = COB_BSWAP_64 (n);
+			n >>= 8 * fsiz;	/* shift with sign */
+		} else {
+			num_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
+			n = COB_BSWAP_64 (n);
+		}
+	} else {
+		if (COB_FIELD_HAVE_SIGN (f)) {
+			num_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
+			n >>= 8 * fsiz;	/* shift with sign */
+		} else {
+			num_byte_memcpy ((unsigned char *)&n, f->data, f->size);
+		}
+	}
+#else	/* WORDS_BIGENDIAN */
+	if (COB_FIELD_HAVE_SIGN (f)) {
+		num_byte_memcpy ((unsigned char *)&n, f->data, f->size);
+		n >>= 8 * fsiz;	/* shift with sign */
+	} else {
+		num_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
+	}
+#endif	/* WORDS_BIGENDIAN */
+	return n;
+}
+
+static unsigned long long
+cob_binary_get_uint64 (const cob_field * const f)
+{
+	unsigned long long	n = 0;
+	size_t			fsiz = 8 - f->size;
+
+#ifndef WORDS_BIGENDIAN
+	if (COB_FIELD_BINARY_SWAP (f)) {
+		num_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
+		n = COB_BSWAP_64 (n);
+	} else {
+		num_byte_memcpy ((unsigned char *)&n, f->data, f->size);
+	}
+#else	/* WORDS_BIGENDIAN */
+	num_byte_memcpy (((unsigned char *)&n) + fsiz, f->data, f->size);
+#endif	/* WORDS_BIGENDIAN */
+	return n;
+}
+
+static void
+cob_binary_set_uint64 (cob_field *f, unsigned long long n)
+{
+#ifndef WORDS_BIGENDIAN
+	unsigned char	*s;
+
+	if (COB_FIELD_BINARY_SWAP (f)) {
+		n = COB_BSWAP_64 (n);
+		s = ((unsigned char *)&n) + 8 - f->size;
+	} else {
+		s = (unsigned char *)&n;
+	}
+	num_byte_memcpy (f->data, s, f->size);
+#else	/* WORDS_BIGENDIAN */
+	num_byte_memcpy (f->data, ((unsigned char *)&n) + 8 - f->size, f->size);
+#endif	/* WORDS_BIGENDIAN */
+}
+
+static void
+cob_binary_set_int64 (cob_field *f, long long n)
+{
+#ifndef WORDS_BIGENDIAN
+	unsigned char	*s;
+
+	if (COB_FIELD_BINARY_SWAP (f)) {
+		n = COB_BSWAP_64 (n);
+		s = ((unsigned char *)&n) + 8 - f->size;
+	} else {
+		s = (unsigned char *)&n;
+	}
+	num_byte_memcpy (f->data, s, f->size);
+#else	/* WORDS_BIGENDIAN */
+	num_byte_memcpy (f->data, ((unsigned char *)&n) + 8 - f->size, f->size);
+#endif	/* WORDS_BIGENDIAN */
+}
+
 /*
  * Decimal number
  */
@@ -225,9 +342,11 @@ cob_decimal_set_double (cob_decimal *d, const double v)
 static double
 cob_decimal_get_double (cob_decimal *d)
 {
-	int	n = d->scale;
-	double	v = mpz_get_d (d->value);
+	double	v;
+	int	n;
 
+	v = mpz_get_d (d->value);
+	n = d->scale;
 	for (; n > 0; n--) {
 		v /= 10;
 	}
@@ -242,12 +361,13 @@ cob_decimal_get_double (cob_decimal *d)
 static void
 cob_decimal_set_display (cob_decimal *d, cob_field *f)
 {
+	unsigned char	*data;
+	size_t		size;
 	int		sign;
-	size_t		size = COB_FIELD_SIZE (f);
-	unsigned char	*data = COB_FIELD_DATA (f);
 	unsigned int	n;
-	unsigned char	buff[64];
 
+	data = COB_FIELD_DATA (f);
+	size = COB_FIELD_SIZE (f);
 	if (unlikely(*data == 255)) {
 		mpz_ui_pow_ui (d->value, 10, size);
 		d->scale = COB_FIELD_SCALE(f);
@@ -274,9 +394,9 @@ cob_decimal_set_display (cob_decimal *d, cob_field *f)
 		}
 		mpz_set_ui (d->value, n);
 	} else {
-		own_memcpy (buff, data, size);
-		buff[size] = 0;
-		mpz_set_str (d->value, (char *)buff, 10);
+		memcpy (num_buff_ptr, data, size);
+		num_buff_ptr[size] = 0;
+		mpz_set_str (d->value, (char *)num_buff_ptr, 10);
 	}
 
 	/* set sign and scale */
@@ -290,18 +410,19 @@ cob_decimal_set_display (cob_decimal *d, cob_field *f)
 static int
 cob_decimal_get_display (cob_decimal *d, cob_field *f, const int opt)
 {
-	int		diff;
-	int		sign = mpz_sgn (d->value);
+	unsigned char	*data;
 	size_t		size;
-	unsigned char	*data = COB_FIELD_DATA (f);
-	unsigned char	buff[COB_SMALL_BUFF];
+	int		diff;
+	int		sign;
 
 	/* build string */
+	sign = mpz_sgn (d->value);
 	mpz_abs (d->value, d->value);
-	mpz_get_str ((char *)buff, 10, d->value);
-	size = strlen ((char *)buff);
+	mpz_get_str ((char *)num_buff_ptr, 10, d->value);
+	size = strlen ((char *)num_buff_ptr);
 
 	/* store number */
+	data = COB_FIELD_DATA (f);
 	diff = (int)(COB_FIELD_SIZE (f) - size);
 	if (unlikely(diff < 0)) {
 		/* overflow */
@@ -314,11 +435,11 @@ cob_decimal_get_display (cob_decimal *d, cob_field *f, const int opt)
 		}
 
 		/* othersize, truncate digits */
-		own_memcpy (data, buff - diff, COB_FIELD_SIZE (f));
+		memcpy (data, num_buff_ptr - diff, COB_FIELD_SIZE (f));
 	} else {
 		/* no overflow */
-		own_memset (data, '0', (size_t)diff);
-		own_memcpy (data + diff, buff, size);
+		memset (data, '0', (size_t)diff);
+		memcpy (data + diff, num_buff_ptr, size);
 	}
 
 	cob_put_sign (f, sign);
@@ -336,7 +457,7 @@ cob_decimal_set_binary (cob_decimal *d, cob_field *f)
 	} else {
 		mpz_set_ui (d->value, cob_binary_get_uint64 (f));
 	}
-#elif	COB_EXPERIMENTAL
+#elif	defined(COB_EXPERIMENTAL)
 	if (COB_FIELD_HAVE_SIGN (f)) {
 		mpz_set_sll (d->value, cob_binary_get_int64 (f));
 	} else {
@@ -349,9 +470,9 @@ cob_decimal_set_binary (cob_decimal *d, cob_field *f)
 
 	if (f->size <= 4) {
 		if (COB_FIELD_HAVE_SIGN (f)) {
-			mpz_set_si (d->value, cob_binary_get_int (f));
+			mpz_set_si (d->value, (int)cob_binary_get_int64 (f));
 		} else {
-			mpz_set_ui (d->value, (unsigned int) cob_binary_get_int (f));
+			mpz_set_ui (d->value, (unsigned int) cob_binary_get_uint64 (f));
 		}
 	} else {
 		if (COB_FIELD_HAVE_SIGN (f)) {
@@ -391,7 +512,7 @@ cob_decimal_get_binary (cob_decimal *d, cob_field *f, const int opt)
 #endif
 
 	if (unlikely(mpz_size (d->value) == 0)) {
-		own_memset (f->data, 0, f->size);
+		memset (f->data, 0, f->size);
 		return 0;
 	}
 	overflow = 0;
@@ -413,6 +534,12 @@ cob_decimal_get_binary (cob_decimal *d, cob_field *f, const int opt)
 		/* TRUNC_ON_OVERFLOW is only set for binary_truncate */
 		if (opt & COB_STORE_TRUNC_ON_OVERFLOW) {
 			mpz_tdiv_r (d->value, d->value, cob_mpze10[digits]);
+/*
+		}
+*/
+/* RXW */
+		} else {
+			mpz_fdiv_r_2exp (d->value, d->value, (f->size * 8));
 		}
 	} else if (opt && cob_current_module->flag_binary_truncate) {
 		if (mpz_cmpabs (d->value, cob_mpze10[digits]) >= 0) {
@@ -424,6 +551,12 @@ cob_decimal_get_binary (cob_decimal *d, cob_field *f, const int opt)
 			/* TRUNC_ON_OVERFLOW is only set for binary_truncate */
 			if (opt & COB_STORE_TRUNC_ON_OVERFLOW) {
 				mpz_tdiv_r (d->value, d->value, cob_mpze10[digits]);
+/*
+			}
+*/
+/* RXW */
+			} else {
+				mpz_fdiv_r_2exp (d->value, d->value, (f->size * 8));
 			}
 		}
 	}
@@ -433,7 +566,7 @@ cob_decimal_get_binary (cob_decimal *d, cob_field *f, const int opt)
 	} else {
 		cob_binary_set_int64 (f, mpz_get_si (d->value));
 	}
-#elif	COB_EXPERIMENTAL
+#elif	defined(COB_EXPERIMENTAL)
 	if (!sign || overflow) {
 		cob_binary_set_uint64 (f, mpz_get_ull (d->value));
 	} else {
@@ -442,9 +575,9 @@ cob_decimal_get_binary (cob_decimal *d, cob_field *f, const int opt)
 #else
 	if (f->size <= 4) {
 		if (!sign || overflow) {
-			cob_binary_set_uint64 (f, mpz_get_ui (d->value));
+			cob_binary_set_uint64 (f, (unsigned long long)mpz_get_ui (d->value));
 		} else {
-			cob_binary_set_int64 (f, mpz_get_si (d->value));
+			cob_binary_set_int64 (f, (long long)mpz_get_si (d->value));
 		}
 	} else {
 		mpz_fdiv_r_2exp (cob_mpzt, d->value, 32);
@@ -478,19 +611,22 @@ cob_packed_get_sign (const cob_field *f)
 {
 	unsigned char *p;
 
-	p = f->data + (COB_FIELD_DIGITS(f) / 2);
+	if (!COB_FIELD_HAVE_SIGN (f)) {
+		return 0;
+	}
+	p = f->data + f->size - 1;
 	return ((*p & 0x0f) == 0x0d) ? -1 : 1;
 }
 
 static void
 cob_add_packed (cob_field *f, int val)
 {
-	int		sign;
 	unsigned char	*p;
+	int		sign;
 	int		ndigs;
-	unsigned int	msn;
 	int		tval;
 	int		carry = 0;
+	unsigned int	msn;
 	unsigned int	subtr = 0;
 	unsigned int	zeroes = 0;
 	unsigned int	origdigs;
@@ -499,7 +635,7 @@ cob_add_packed (cob_field *f, int val)
 	if (ndigs <= 0) {
 		return;
 	}
-	sign = COB_FIELD_HAVE_SIGN (f) ? cob_packed_get_sign (f) : 0;
+	sign = cob_packed_get_sign (f);
 	msn = 1 - (COB_FIELD_SCALE(f) % 2);
 
 	/* -x +v = -(x - v), -x -v = -(x + v) */
@@ -569,13 +705,16 @@ cob_add_packed (cob_field *f, int val)
 static void
 cob_decimal_set_packed (cob_decimal *d, cob_field *f)
 {
-	unsigned char	*p = f->data;
-	int		digits = COB_FIELD_DIGITS(f);
+	unsigned char	*p;
+	int		digits;
 	int		sign;
 	unsigned int	val;
 	unsigned int	valseen;
 
-	sign = COB_FIELD_HAVE_SIGN (f) ? cob_packed_get_sign (f) : 0;
+	p = f->data;
+	/* Fixme */
+	digits = COB_FIELD_DIGITS (f);
+	sign = cob_packed_get_sign (f);
 
 	if (digits % 2 == 0) {
 		val = *p & 0x0f;
@@ -634,25 +773,28 @@ cob_decimal_set_packed (cob_decimal *d, cob_field *f)
 static int
 cob_decimal_get_packed (cob_decimal *d, cob_field *f, const int opt)
 {
-	int		diff;
-	int		sign = mpz_sgn (d->value);
+	unsigned char	*data;
+	unsigned char	*p;
+	unsigned char	*q;
 	size_t		size;
 	size_t		n;
 	size_t		i;
-	int		digits = COB_FIELD_DIGITS(f);
-	unsigned char	*data = f->data;
-	unsigned char	*p;
-	unsigned char	*q;
+	int		diff;
+	int		sign;
+	int		digits;
 	unsigned char	x;
-	unsigned char	buff[COB_MEDIUM_BUFF];
 
 	/* build string */
+	sign = mpz_sgn (d->value);
 	mpz_abs (d->value, d->value);
-	mpz_get_str ((char *)buff, 10, d->value);
-	size = strlen ((char *)buff);
+	mpz_get_str ((char *)num_buff_ptr, 10, d->value);
+	size = strlen ((char *)num_buff_ptr);
 
 	/* store number */
-	q = buff;
+	data = f->data;
+	/* Fixme */
+	digits = COB_FIELD_DIGITS(f);
+	q = num_buff_ptr;
 	diff = (int)(digits - size);
 	if (diff < 0) {
 		/* overflow */
@@ -666,7 +808,7 @@ cob_decimal_get_packed (cob_decimal *d, cob_field *f, const int opt)
 		q += size - digits;
 		size = digits;
 	}
-	own_memset (data, 0, f->size);
+	memset (data, 0, f->size);
 	p = data + (digits / 2) - (size / 2);
 	diff = 1 - (int)(size % 2);
 	for (i = diff, n = 0; i < size + diff; i++, n++) {
@@ -678,15 +820,13 @@ cob_decimal_get_packed (cob_decimal *d, cob_field *f, const int opt)
 		}
 	}
 
-	p = data + (digits / 2);
+	p = f->data + f->size - 1;
 	if (!COB_FIELD_HAVE_SIGN (f)) {
-		*p |= 0x0f;
+		*p = (*p & 0xf0) | 0x0f;
+	} else if (sign < 0) {
+		*p = (*p & 0xf0) | 0x0d;
 	} else {
-		if (sign < 0) {
-			*p |= 0x0d;
-		} else {
-			*p |= 0x0c;
-		}
+		*p = (*p & 0xf0) | 0x0c;
 	}
 
 	return 0;
@@ -695,11 +835,11 @@ cob_decimal_get_packed (cob_decimal *d, cob_field *f, const int opt)
 void
 cob_set_packed_zero (cob_field *f)
 {
-	own_memset (f->data, 0, f->size);
+	memset (f->data, 0, f->size);
 	if (!COB_FIELD_HAVE_SIGN (f)) {
-		*(f->data + f->size - 1) |= 0x0f;
+		*(f->data + f->size - 1) = 0x0f;
 	} else {
-		*(f->data + f->size - 1) |= 0x0c;
+		*(f->data + f->size - 1) = 0x0c;
 	}
 }
 
@@ -710,13 +850,13 @@ cob_set_packed_int (cob_field *f, const int val)
 	size_t		sign = 0;
 	int		n;
 
-	if ( val < 0) {
+	if (val < 0) {
 		n = -val;
 		sign = 1;
 	} else {
 		n = val;
 	}
-	own_memset (f->data, 0, f->size);
+	memset (f->data, 0, f->size);
 	p = f->data + f->size - 1;
 	*p = (n % 10) << 4;
 	if (!COB_FIELD_HAVE_SIGN (f)) {
@@ -731,6 +871,7 @@ cob_set_packed_int (cob_field *f, const int val)
 	for (; n && p >= f->data; n /= 100, p--) {
 		*p = packed_bytes[n % 100];
 	}
+	/* Fixme */
 	if ((COB_FIELD_DIGITS(f) % 2) == 0) {
 		*(f->data) &= 0x0f;
 	}
@@ -741,8 +882,8 @@ cob_set_packed_int (cob_field *f, const int val)
 void
 cob_decimal_set_field (cob_decimal *d, cob_field *f)
 {
-	float	fval;
 	double	dval;
+	float	fval;
 
 	switch (COB_FIELD_TYPE (f)) {
 	case COB_TYPE_NUMERIC_BINARY:
@@ -872,7 +1013,10 @@ cob_decimal_div (cob_decimal *d1, cob_decimal *d2)
 		cob_set_exception (COB_EC_SIZE_ZERO_DIVIDE);
 		return;
 	}
-
+	if (unlikely(mpz_sgn (d1->value) == 0)) {
+		d1->scale = 0;
+		return;
+	}
 	d1->scale -= d2->scale;
 	shift_decimal (d1, 37 + ((d1->scale < 0) ? -d1->scale : 0));
 	mpz_tdiv_q (d1->value, d1->value, d2->value);
@@ -909,11 +1053,12 @@ cob_decimal_cmp (cob_decimal *d1, cob_decimal *d2)
 static int
 display_add_int (unsigned char *data, const size_t size, unsigned int n)
 {
-	unsigned char	*sp = data + size;
+	unsigned char	*sp;
 	size_t		carry = 0;
 	int		i;
 	int		is;
 
+	sp = data + size;
 	while (n > 0) {
 		i = n % 10;
 		n /= 10;
@@ -956,10 +1101,11 @@ display_add_int (unsigned char *data, const size_t size, unsigned int n)
 static int
 display_sub_int (unsigned char *data, const size_t size, unsigned int n)
 {
-	unsigned char	*sp = data + size;
+	unsigned char	*sp;
 	size_t		carry = 0;
 	int		i;
 
+	sp = data + size;
 	while (n > 0) {
 		i = n % 10;
 		n /= 10;
@@ -994,17 +1140,20 @@ display_sub_int (unsigned char *data, const size_t size, unsigned int n)
 static int
 cob_display_add_int (cob_field *f, int n)
 {
-	int		sign;
+	unsigned char	*data;
 	size_t		osize;
 	size_t		i;
-	unsigned char	*data = COB_FIELD_DATA (f);
-	size_t		size = COB_FIELD_SIZE (f);
-	int		scale = COB_FIELD_SCALE (f);
+	size_t		size;
+	int		scale;
+	int		sign;
 	unsigned char	tfield[64];
 
-	osize = size;
-	own_memcpy (tfield, data, osize);
+	data = COB_FIELD_DATA (f);
+	size = COB_FIELD_SIZE (f);
+	scale = COB_FIELD_SCALE (f);
 	sign = cob_get_sign (f);
+	osize = size;
+	memcpy (tfield, data, osize);
 	/* -x + n = -(x - n) */
 	if (sign < 0) {
 		n = -n;
@@ -1035,7 +1184,7 @@ cob_display_add_int (cob_field *f, int n)
 		/* add n to the field */
 		if (display_add_int (data, size, n) != 0) {
 			/* if there was an overflow, recover the last value */
-			own_memcpy (data, tfield, osize);
+			memcpy (data, tfield, osize);
 			goto overflow;
 		}
 	} else if (n < 0) {
@@ -1098,7 +1247,7 @@ cob_add_int (cob_field *f, const int n)
 		mpz_set_si (cob_d2.value, n);
 		cob_d2.scale = 0;
 		if (cob_d1.scale) {
-			mpz_ui_pow_ui (cob_mexp, 10, cob_d1.scale);
+			mpz_ui_pow_ui (cob_mexp, 10, (unsigned int)cob_d1.scale);
 			mpz_mul (cob_d2.value, cob_d2.value, cob_mexp);
 			cob_d2.scale = cob_d1.scale;
 		}
@@ -1117,9 +1266,10 @@ cob_sub_int (cob_field *f, const int n)
 }
 
 int
-cob_div_quotient (cob_field *dividend, cob_field *divisor, cob_field *quotient, const int opt)
+cob_div_quotient (cob_field *dividend, cob_field *divisor,
+		  cob_field *quotient, const int opt)
 {
-	int ret;
+	int	ret;
 
 	cob_decimal_set_field (&cob_d1, dividend);
 	cob_decimal_set_field (&cob_d2, divisor);
@@ -1181,14 +1331,15 @@ cob_numeric_cmp (cob_field *f1, cob_field *f2)
 int
 cob_cmp_packed (cob_field *f, int n)
 {
-	int			sign;
+	static int		lastval = 0;
+
+	unsigned char		*p;
 	size_t			size;
 	size_t			inc = 0;
-	static int		lastval = 0;
-	unsigned char		*p;
+	int			sign;
 	unsigned char		val1[20];
 
-	sign = COB_FIELD_HAVE_SIGN (f) ? cob_packed_get_sign (f) : 0;
+	sign = cob_packed_get_sign (f);
 	/* Field positive, value negative */
 	if (sign >= 0 && n < 0) {
 		return 1;
@@ -1215,7 +1366,7 @@ cob_cmp_packed (cob_field *f, int n)
 		if (n < 0) {
 			n = -n;
 		}
-		own_memset (&packed_value[14], 0, 6);
+		memset (&packed_value[14], 0, 6);
 		if (n) {
 			p = &packed_value[19];
 			*p =  (n % 10) << 4;
@@ -1256,6 +1407,7 @@ cob_init_numeric (void)
 		mpz_init (cob_mpze10[i]);
 		mpz_ui_pow_ui (cob_mpze10[i], 10, i);
 	}
+	num_buff_ptr = cob_malloc (2048);
 	memset (packed_value, 0, sizeof(packed_value));
 }
 
@@ -1264,9 +1416,9 @@ cob_init_numeric (void)
 int
 cob_cmp_numdisp (const unsigned char *data, const size_t size, const int n)
 {
-	int			val = 0;
-	size_t			inc;
 	const unsigned char	*p;
+	size_t			inc;
+	int			val = 0;
 
 	p = data;
 	for (inc = 0; inc < size; inc++, p++) {
@@ -1278,9 +1430,9 @@ cob_cmp_numdisp (const unsigned char *data, const size_t size, const int n)
 int
 cob_cmp_long_numdisp (const unsigned char *data, const size_t size, const int n)
 {
+	const unsigned char	*p;
 	long long		val = 0;
 	size_t			inc;
-	const unsigned char	*p;
 
 	p = data;
 	for (inc = 0; inc < size; inc++, p++) {
@@ -1498,9 +1650,9 @@ cob_get_long_ebcdic_sign (const unsigned char *p, long long *val)
 int
 cob_cmp_sign_numdisp (const unsigned char *data, const size_t size, const int n)
 {
+	const unsigned char	*p;
 	int			val = 0;
 	size_t			inc;
-	const unsigned char	*p;
 
 	p = data;
 	for (inc = 0; inc < size - 1; inc++, p++) {
@@ -1529,9 +1681,9 @@ cob_cmp_sign_numdisp (const unsigned char *data, const size_t size, const int n)
 int
 cob_cmp_long_sign_numdisp (const unsigned char *data, const size_t size, const int n)
 {
+	const unsigned char	*p;
 	long long		val = 0;
 	size_t			inc;
-	const unsigned char	*p;
 
 	p = data;
 	for (inc = 0; inc < size - 1; inc++, p++) {
