@@ -1,7 +1,6 @@
-##!/usr/bin/perl -s
 #
-# Copyright (C) 2001-2007 Keisuke Nishida
-# Copyright (C) 2007 Roger While
+# Copyright (C) 2001-2009 Keisuke Nishida
+# Copyright (C) 2007-2009 Roger While
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,8 +19,16 @@
 
 my $opt = shift;
 
-my $compile = "cobc -std=cobol85 -x $opt";
-my $compile_module = "cobc -std=cobol85 -m $opt";
+my $compile;
+my $compile_module;
+
+if ($opt) {
+	$compile = "cobc -std=cobol85 -x $opt";
+	$compile_module = "cobc -std=cobol85 -m $opt";
+} else {
+	$compile = "cobc -std=cobol85 -x";
+	$compile_module = "cobc -std=cobol85 -m";
+}
 
 my $num_progs = 0;
 my $test_skipped = 0;
@@ -34,19 +41,27 @@ my $total_fail = 0;
 my $total_deleted = 0;
 my $total_inspect = 0;
 my $total_ok = 0;
+my $ret = 0;
 
 $ENV{"COB_SWITCH_1"} = "ON";
 $ENV{"COB_SWITCH_2"} = "OFF";
 
 $skip{SQ207M} = 1;
+$skip{SG102A} = 1;
+$skip{SG103A} = 1;
+$skip{SG201A} = 1;
+$skip{SG202A} = 1;
+$skip{SG203A} = 1;
+$skip{OBNC1M} = 1;
+$skip{OBNC2M} = 1;
 
 open (LOG, "> report.txt") or die;
 print LOG "Filename    total pass fail deleted inspect\n";
 print LOG "--------    ----- ---- ---- ------- -------\n";
 
 foreach $in (glob("lib/*.CBL")) {
-  print "$compile_module $in\n";
-  system ("$compile_module $in");
+	print "$compile_module $in\n";
+	system ("$compile_module $in");
 }
 
 foreach $in (sort (glob("*.{CBL,SUB}"))) {
@@ -64,14 +79,23 @@ foreach $in (sort (glob("*.{CBL,SUB}"))) {
     print LOG "  ----- test skipped -----\n";
   } else {
     $num_progs++;
-    $copy = ($exe =~ /^SM/) ? "-I ../copy " : "";
-    print "$compile $copy$in && $cmd\n";
-    if (system ("$compile $copy$in") != 0) {
+    $copy = ($exe =~ /^SM/) ? "-I ../copy" : "";
+    print "$compile $copy $in && $cmd\n";
+    if ($in eq "SM206A.CBL") {
+      $ret = system ("$compile -fdebugging-line $copy $in");
+    } else {
+      $ret = system ("$compile $copy $in");
+    }
+    if ($ret != 0) {
       $compile_error++;
       print LOG "  ===== compile error =====\n";
     } else {
       if ($in =~ /\.CBL/) {
-	system ("rm -f XXXXX*; test -n \"$ENV{DB_HOME}\" && rm -f $ENV{DB_HOME}/XXXXX*");
+	if ($ENV{'DB_HOME'}) {
+		system ("rm -f XXXXX*; rm -f $ENV{'DB_HOME'}/XXXXX*");
+	} else {
+		system ("rm -f XXXXX*");
+	}
       }
       if (system ("$cmd > $exe.out") != 0) {
 	$execute_error++;
